package cn.com.ctop.kuaishou.modules.batch.service.impl;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouCreative;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouVideoGet;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaiShouCreativeMapper;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaiShouVideoGetMapper;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouCreativeService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 快手-创意信息
 * @Author: jeecg-boot
 * @Date: 2019-07-23
 * @Version: V1.0
 */
@Slf4j
@Service
public class KuaiShouCreativeServiceImpl extends ServiceImpl<KuaiShouCreativeMapper, KuaiShouCreative> implements IKuaiShouCreativeService {
    @Autowired
    private KuaiShouCreativeMapper kuaiShouCreativeMapper;
    @Autowired
    private KuaiShouVideoGetMapper kuaiShouVideoGetMapper;

    @Override
    public void replaceBatch(List<KuaiShouCreative> creatives) {
        kuaiShouCreativeMapper.replaceBatch(creatives);
    }

    @Override
    public String getCodeByCreativeId(Long creativeId) {
        QueryWrapper<KuaiShouCreative> creativeQueryWrapper = new QueryWrapper<>();
        creativeQueryWrapper.eq("creative_id", creativeId);
        creativeQueryWrapper.orderByDesc("create_time");
        creativeQueryWrapper.last("limit 1");
        KuaiShouCreative kuaiShouCreative = kuaiShouCreativeMapper.selectOne(creativeQueryWrapper);
        if (!Check.isNull(kuaiShouCreative)) {
            String photoId = kuaiShouCreative.getPhotoId();
            if (!Check.isNull(photoId)) {
                QueryWrapper<KuaiShouVideoGet> videoGetQueryWrapper = new QueryWrapper<>();
                videoGetQueryWrapper.eq("photo_id", photoId);
                videoGetQueryWrapper.orderByDesc("create_time");
                videoGetQueryWrapper.last("limit 1");
                KuaiShouVideoGet kuaiShouVideoGet = kuaiShouVideoGetMapper.selectOne(videoGetQueryWrapper);
                if (!Check.isNull(kuaiShouVideoGet)) {
                    return kuaiShouVideoGet.getSignature();
                }
            }
        }
        return null;
    }

    /**
     * 创意条数
     *
     * @param accountId
     * @param unitId
     * @return
     */
    @Override
    public Integer checkCreativeCount(Long accountId, Long unitId) {
        return kuaiShouCreativeMapper.checkCreativeCount(accountId, unitId);
    }

    @Override
    public void getProgramCreative(CtopOauthToken token, int page) {

        log.info("获取视频数据，accountId：{}", token.getAccountId());
        String url = "https://ad.e.kuaishou.com/rest/openapi/v2/creative/advanced/program/list";
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", token.getAccountId());
        param.put("page_size", 500);
        param.put("page", page);

        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手视频列表数据异常:{},accountId:{}", message, token.getAccountId());
            return;
        }
        JSONObject dataJson = resultJson.getJSONObject("data");
        if (Check.isNull(dataJson)) {
            return;
        }
        JSONArray details = dataJson.getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }

        for (int i = 0; i < details.size(); i++) {
            JSONObject jsonObject = details.getJSONObject(i);
            if (Check.isNull(jsonObject)) {
                continue;
            }

            String updateUrl = "https://ad.e.kuaishou.com/rest/openapi/v2/creative/advanced/program/update";

            Map<String, String> headers1 = new HashMap<>();
            headers1.put("Content-Type", "application/json");
            headers1.put("Access-Token", token.getAccessToken());
            Map<String, Object> param1 = new HashMap<>();
            param1.put("advertiser_id", token.getAccountId());
            param1.put("unit_id", jsonObject.getLong("unit_id"));

            String clickTrackUrl = jsonObject.getString("click_url");
            if (Check.isNull(clickTrackUrl)) {
                continue;
            }


            String a = "";

            if (clickTrackUrl.contains("&pass_through")) {
                if (clickTrackUrl.contains("&ua=__UA__&oaid=__OAID__")) {
                    continue;
                } else {
                    String[] split = clickTrackUrl.split("&pass_through");
                    a = split[0] + "&ua=__UA__&oaid=__OAID__" + "&pass_through" + split[1];
                }

            } else {
                a = clickTrackUrl + "&ua=__UA__&oaid=__OAID__";
            }

            param1.put("click_url", a);
            String result1 = HttpUtils.httpPostRequest(updateUrl, param1, headers1);
            JSONObject resultJson1 = JSONObject.parseObject(result1);
            Integer code1 = resultJson1.getInteger("code");
            if (code1 == 0) {
                log.info("修改成功,accountId:{},unitId:{}", token.getAccountId(), jsonObject.getLong("unit_id"));

            } else {
                log.info("修改失败,accountId:{},unitId:{}，原因：{}", token.getAccountId(), jsonObject.getLong("unit_id"), resultJson1.getString("message"));
            }


        }

        getProgramCreative(token, page + 1);

    }
}
